﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices.Marshalling;
using System.Text;
using System.Threading.Tasks;

namespace Kompozyt
{
    public interface IPracownik //liść
    {
        void WyświetlInformacje(int głębokość);
    }

    public class Pracownik : IPracownik
    {
        private string imię, nazwisko, stanowisko;
        protected internal bool odwiedzony = false;
        
        public Pracownik(string imię, string nazwisko, string stanowisko)
        {
            this.imię = imię;
            this.nazwisko = nazwisko;
            this.stanowisko = stanowisko;
        }

        public override string ToString()
        {
            return imię + " " + nazwisko + " (" + stanowisko + ")";
        }

        private void zróbWcięcie(int głębokość)
        {
            for(int i = 0; i < głębokość; i++) { Console.Write("   "); }
        }

        public virtual void WyświetlInformacje(int głębokość = 0)
        {
            zróbWcięcie(głębokość);
            Console.WriteLine(ToString());
            odwiedzony = true;
        }

        public virtual void Resetuj()
        {
            odwiedzony = false;
        }
    }

    public class Kierownik : Pracownik //kompozyt
    {
        private List<IPracownik> podwładni = new List<IPracownik>();

        public Kierownik(string imię, string nazwisko, string stanowisko)
            :base(imię, nazwisko, stanowisko)
        { }

        public void DodajPodwładnych(params IPracownik[] podwładni)
        {
            this.podwładni.AddRange(podwładni);
        }

        public override void WyświetlInformacje(int głębokość = 0)
        {
            base.WyświetlInformacje(głębokość);
            if (podwładni.Count > 0)
            {
                Console.WriteLine("Podwładni:");
                foreach (IPracownik podwładny in podwładni)
                {
                    if(!podwładny.odwi)
                    podwładny.WyświetlInformacje(głębokość + 1);
                }
            }
        }
    }
}
